﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace SRF01
{
    public partial class Form1 : Form
    {
        byte SerAddress = 0;
        byte NewAddress = 0;
        byte Seq = 0;
        byte SearchAddr = 1;
        bool PortOpen = false;
        bool Sleeping = false;
        bool GoToSleep = false;
        bool WakeUp = false;
        static SerialPort SerPort;

        // commands starting with X_ are unimplemented uS commands, they exist only for SRF02 compatibility
        enum Commands {
            REAL_IN=0x50,	REAL_CM,		X_REAL_US,		REAL_IN_TX,		REAL_CM_TX,		X_REAL_US_TX,
			FAKE_IN,		FAKE_CM, 		X_FAKE_US,		FAKE_IN_TX,		FAKE_CM_TX,		X_FAKE_US_TX,
			BURST, 		    GET_VER,		GET_RANGE,		GET_STATUS,		SLEEP,			UNLOCK,		 	
			SET_ADVANCED,	CLR_ADVANCED,	BAUD19200,		BAUD38400,
};

        public Form1()
        {
            InitializeComponent();
            SerPort = new SerialPort();
            // Get a list of serial port names.
            foreach (string s in SerialPort.GetPortNames())
            {
                Ports.Items.Add(s);
            }
            Range.Text = "Select Port";

        }

        private void Ports_SelectedIndexChanged(object sender, EventArgs e)
        {
            byte[] SerBuf = new byte[10];
            SerPort.PortName = Ports.Text;
            SerPort.Parity = 0;
            SerPort.BaudRate = 9600;
            SerPort.StopBits = StopBits.One;
            SerPort.DataBits = 8;
            SerPort.ReadTimeout = 100;
            SerPort.WriteTimeout = 100;
            SerPort.Open();
            PortOpen = true;
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void Update_CheckedChanged(object sender, EventArgs e)
        {
            int x;
            byte A;

            if (UpdateAddr.Checked)
            {
                x = Address.Text.Length;
                if (x == 1 || x == 2)
                {
                    try
                    {
                        A = Convert.ToByte(Address.Text.Substring(0, x), 10);
                        if (A > 0 && A < 17)
                        {
                            NewAddress = A;
                            Seq = 4;
                        }
                        else
                        {
                            Address.Text = string.Format("{0}", SerAddress);
                            UpdateAddr.Checked = false;
                        }
                    }
                    catch
                    {
                        Address.Text = string.Format("{0}", SerAddress);
                        UpdateAddr.Checked = false;
                    }
                }
            }
        }

        private void Sleep_CheckedChanged(object sender, EventArgs e)
        {
            if (Sleep.Checked)
            {
                GoToSleep = true;
            }
            else
            {
                WakeUp = true;
                Sleeping = false;
            }
        }

        private void SendBreak()
        {
            SerPort.BreakState = true; 
            Thread.Sleep(2);
            SerPort.BreakState = false;
        }

        private void Disgard(byte count)
        {
            byte[] SerBuf = new byte[count+1];
            while (SerPort.BytesToRead < count) ;      
            SerPort.Read(SerBuf, 0, count);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            byte[] SerBuf = new byte[10];
            int x;

            if (!PortOpen) return;

            if (Sleeping)  return;

            if (GoToSleep)
            {
                SendBreak();
                SerBuf[0] = SerAddress;                    // Address
                SerBuf[1] = (byte)Commands.SLEEP;          // send Sleep command
                SerPort.Write(SerBuf, 0, 2);
                Disgard(3);                                // disgard echoed characters from our Tx
                GoToSleep = false;
                Sleeping = true;
                Range.Text = "Sleeping";
                return;
            }

            if (WakeUp)
            {
                SerBuf[0] = 0xff;                           // 0xff is neither address or command, 
                SerPort.Write(SerBuf, 0, 1);                  // so its a good wake up byte to use
                WakeUp = false; 
                Disgard(1);                                  // disgard echoed characters from our Tx
                return;
            }

            if (SerAddress == 0)
            {
                SendBreak();
                SerBuf[0] = SearchAddr;                   // Address
                SerBuf[1] = (byte)Commands.GET_VER;       // Get Revision
                SerPort.Write(SerBuf, 0, 2);
                Address.Text = string.Format("{0}", SearchAddr);
                Address.Update();
                Disgard(3);                               // disgard echoed characters from our Tx

                try { SerPort.Read(SerBuf, 0, 1); }
                catch
                {
                    SearchAddr++;
                    if (SearchAddr > 16) SearchAddr = 1;
                    Range.Text = "Searching";
                    Range.Update();
                    return;
                }

                if (SerBuf[0] < 100)
                {
                    SerAddress = SearchAddr;
                    Range.Text = "Found";
                    Range.Update();
                    Version.Text = string.Format("{0}", SerBuf[0]);
                    Version.Update();
                }
                return;
            }

            if (NewAddress > 0)
            {
                switch (Seq)
                {
                    case 4:
                        SendBreak();
                        SerBuf[0] = SerAddress;                 // Address
                        SerBuf[1] = 0xa0;				    	// start of sequence to change address; 
                        SerPort.Write(SerBuf, 0, 2);
                        Seq = 3;
                        break;
                    case 3:
                        SendBreak();
                        SerBuf[0] = SerAddress;                 // Address
                        SerBuf[1] = 0xaa;				    	// 2nd in sequence to change address; 
                        SerPort.Write(SerBuf, 0, 2);
                        Seq = 2;
                        break;
                    case 2:
                        SendBreak();
                        SerBuf[0] = SerAddress;                 // Address
                        SerBuf[1] = 0xa5;				    	// 3rd in sequence to change address; 
                        SerPort.Write(SerBuf, 0, 2);
                        Seq = 1;
                        break;
                    case 1:
                        SendBreak();
                        SerBuf[0] = SerAddress;                 // Address
                        SerBuf[1] = NewAddress;			    	// new address; 
                        SerPort.Write(SerBuf, 0, 2);
                        Disgard(12);                            // disgard echoed characters from our Tx

                        while (SerPort.BytesToRead < 1) ;       // wait for Response
                        SerPort.Read(SerBuf, 0, 1);
                        NewAddress = SerAddress = 0;            // force search after address change
                        Seq = 0;
                        UpdateAddr.Checked = false;
                        break;
                }
                return;
            }

            SendBreak();
            SerBuf[0] = SerAddress;                    // Address
            SerBuf[1] = (byte)Commands.GET_RANGE;      // Get Range
            SerPort.Write(SerBuf, 0, 2);
            Disgard(3);                                // disgard echoed characters from our Tx
            while (SerPort.BytesToRead < 2) ;          // wait for Range to arrive
            SerPort.Read(SerBuf, 0, 2);
            x = (SerBuf[0] << 8) + SerBuf[1];
            Range.Text = string.Format("{0}cm", x);

            SendBreak();
            SerBuf[0] = SerAddress;                    // Address
            SerBuf[1] = (byte)Commands.GET_STATUS;     // Get Status
            SerPort.Write(SerBuf, 0, 2);
            Disgard(3);                                // disgard echoed characters from our Tx
            while (SerPort.BytesToRead < 1) ;          // wait for Response
            SerPort.Read(SerBuf, 0, 1);
            x = SerBuf[0];
            if ((x & 0x01) == 1) Lock.Text = "Locked";
            else Lock.Text = "No";

            SendBreak();
            SerBuf[0] = SerAddress;                     // Address
            SerBuf[1] = (byte)Commands.REAL_CM;         // Start next ranging
            SerPort.Write(SerBuf, 0, 2);
            Disgard(3);                                 // disgard echoed characters from our Tx
        }
    }
}
